! Lagrange mesh method works for different types of interaction
! S-matrix form for real interaction and complex interaction are different
! but the complex interaction form also works for the real interaction
      module  lagrange_mesh_single_channel
      use mesh
      implicit none
      ! note the delta function structure fi(xj) = \lambda_i^{-1/2} \delta_{ij}
      real*8,dimension(:),allocatable :: lagrange_func
      real*8,dimension(:,:), allocatable :: kinetic_matrix
      real*8,dimension(:,:),allocatable :: l_barrier_matrix
      real*8,dimension(:,:),allocatable :: V_matrix
      real*8,dimension(:),allocatable ::  mesh_rr, mesh_rw
      real*8 :: rmax_rmatrix
      contains

c***********************************************************************
       subroutine initial_lagrange_func (rmatch_rmatrix)
       !
       ! before calling this subroutine please initial rmax_rmatrix and nr
       !
       ! this subroutine is used to initial the Lagrange function
       ! for the purpose to reconstruct the scattering wave function

       !!!!!! please gives the matching radius
c***********************************************************************
       use gauss
       use mesh
       use precision
       implicit none
       integer :: ir
       real*8 :: rmatch_rmatrix
       rmax_rmatrix=rmatch_rmatrix

       if(allocated(lagrange_func)) deallocate(lagrange_func)
       if(allocated(mesh_rr)) deallocate(mesh_rr)
       if(allocated(mesh_rw)) deallocate(mesh_rw)

       allocate(lagrange_func(1:nr))
       allocate(mesh_rr(1:nr))
       allocate(mesh_rw(1:nr))

       call gauleg(nr,0.0_dpreal,1.0_dpreal,mesh_rr,mesh_rw)

       do ir=1, nr
         lagrange_func(ir) = 1/ sqrt(rmax_rmatrix * mesh_rw(ir))
       end do

       end subroutine
c-----------------------------------------------------------------------
c***********************************************************************
       subroutine T_operator(mu)
       ! this subroutine is used to compute the kinetic and Bloch operator
       ! in the matrix form
       ! the formulas in this subroutine used the forms of Sofia's notes of
       ! talent course
c***********************************************************************
       use precision
       use constants
       implicit none
       integer :: ir, irp
       real*8 :: f1,f2,f3
       real*8 ::  xi, xj
       real*8 :: mu


       if(allocated(kinetic_matrix)) deallocate(kinetic_matrix)
       allocate(kinetic_matrix(1:nr, 1:nr))

       do ir=1 , nr
         do irp=1, nr

            f1=0.0_dpreal
            f2=0.0_dpreal
            f3=0.0_dpreal
            xi= mesh_rr(ir)
            xj= mesh_rr(irp)

            if(ir==irp) then
               f1= hbarc**2 / (6.0_dpreal * rmax_rmatrix**2 * xi * (1-xi) * mu)
               f2= 4.0_dpreal * nr * (nr+1) + 3.0
               f3= (1.0_dpreal - 6.0_dpreal *xi )/( xi * (1-xi) )
               kinetic_matrix(ir,irp)= f1 * ( f2 + f3 )
            else
               f1= (-1.0_dpreal)**(ir+irp) * hbarc**2 /2.0_dpreal/rmax_rmatrix/rmax_rmatrix/
     +             sqrt( xi * xj * (1-xi) * (1-xj) ) / mu
               f2= nr * (nr+1) + 1.0 + ( xi + xj - 2.0_dpreal * xi * xj )/( (xi-xj)**2 )
               f3= 1.0_dpreal/(1.0_dpreal-xi)  + 1.0_dpreal/(1.0_dpreal -xj)


               kinetic_matrix(ir,irp)= f1* (f2-f3)
            end if
         end do
       end do



       end subroutine
c-----------------------------------------------------------------------
c***********************************************************************
       subroutine Bloch_bound(mu,Bc,B_operator)
       ! this subroutine is used to compute the kinetic and Bloch operator
       ! in the matrix form
       ! note for the bound state the Bloch operator is not zero
       ! the formulas in this subroutine used the forms of Sofia's notes of
       ! talent course
c***********************************************************************
       use precision
       use constants
       implicit none
       integer :: ir, irp
       real*8 ::  xi, xj
       real*8 :: mu
       real*8 :: Bc
       real*8 :: f1
       real*8,dimension(1:nr, 1:nr) :: B_operator


       B_operator=0.0_dpreal
       f1=-hbarc**2 * Bc /2.0_dpreal/mu/ rmax_rmatrix/rmax_rmatrix

       do ir=1 , nr
         do irp=1, nr
            xi= mesh_rr(ir)
            xj= mesh_rr(irp)
            B_operator(ir,irp) = f1 * (-1.0_dpreal)**(ir+irp) / sqrt( xi*xj*(1.0-xi)*(1.0-xj) )
         end do
       end do


       end subroutine
c-----------------------------------------------------------------------
c***********************************************************************
       subroutine centrifugal_barrier (l,mu)
       ! this subroutine is used to compute centrifugal_barrier in
       ! lagrange mesh basis
       ! V_l = \frac{\hbarc^2 l (l+1)}{2 * mu * r}
       ! input : mu  in MeV
       !         l
c***********************************************************************
       use constants
       use precision
       implicit none
       integer :: l , ir
       real*8 :: mu ,xi
       if(allocated(l_barrier_matrix)) deallocate(l_barrier_matrix)
       allocate(l_barrier_matrix(1:nr, 1:nr))
       l_barrier_matrix=0.0_dpreal

       do ir=1 , nr
         xi = mesh_rr(ir)
         l_barrier_matrix(ir,ir) = ( hbarc**2 * l * (l+1) )/ ( 2.0_dpreal * mu * rmax_rmatrix * rmax_rmatrix * xi * xi  )
       end do

       end subroutine
c-----------------------------------------------------------------------
c***********************************************************************
       subroutine lagrange_V(Vpot)
       ! this subroutine is used to compute the V-matrix in the Lagrange
       ! mesh basis
       ! input V is given in the mesh point in the step of hcm
       ! this subroutine interpolate the Vpot to give the correct mesh sets
c***********************************************************************
       use interpolation
       use precision
       implicit none
       real*8,dimension(0:irmatch),intent(in) :: vpot
       integer :: ir,irr
       real*8 ::  r
       logical :: nonlocal
       complex*16, dimension(1:nr,1:nr) :: nlpot

       if(allocated(V_matrix)) deallocate(V_matrix)
       allocate(V_matrix(1:nr,1:nr))
       V_matrix=0.0_dpreal

       do ir = 1, nr
         r = rmax_rmatrix*  mesh_rr(ir)
         V_matrix(ir,ir) = FFR4(r/hcm, vpot, 1+irmatch)
       end do


       end subroutine
c-----------------------------------------------------------------------

      end module
